C      PROGRAM DIFSUPA
C      based on difsupnew.f   and  additionally calculate the average 
C      value of ddif and rdif and the first momentom. 
C      Configurations are obtained from input files, and each conf is
C      then fourier transformed to obtain the scattering intensity.
C      Spherical averaged intensity as well as intensities along  
C      (100), (110) and (111) are calculated around (000). This
C      uncorrected (for background) is output to FOUT file.
C      Spherically averaged intensity is also calculated around the
C      superlattice reflection. 
C      Background intensity due to the terminal phases is subtracted;
C      a characteristic inverse length k1 is calculated using this 
C      corrected intensity profile, and scaling is done using k1.
C      This corrected intensity and scaled intensity data is output
C      to the file FOUTD and FOUTS for the fundamental and superlattice
C      spots respectively.
C      
C      Configuration at any instant is in ID(X,Y,Z); an equivalent
C      1-D array ID1D(N) is also used. The program also uses LOCV(N,K)
C      storing the K-th component of the location of the N-th vacancy.
C
C      Modified 01/07/2001 to calculate superlattice intensity around
C      each {100} positions, in addition to the average value
C      <100> already calculated.

      SUBROUTINE DIFSUPS(IT,NL,ID,CONCB,BID,TIME)

      IMPLICIT NONE

      REAL*8 PI
      PARAMETER (PI=3.1415927)

      INTEGER*4 IT, NL, NLCB, NLCB2

C     AID, BID and VID are the numbers used for identifying the
C     atoms at lattice sites.
      INTEGER*2 BID

C     Our program needs these following configuration-defining
C         variables to be up-to-date.
C     ID(K1,K2,K3) = Identity of atom occupying the position
C         (K1, K2, K3). 1 if it is B; 0 if it is A; -1 if it is V.
C     LOCV(I,N) stores the I-th coordinate (position)
C         of the N-th vacancy.
      INTEGER*2 ID(NL,NL,NL)
      REAL*8 CONCB
      REAL*4 S1(0:63,0:63,0:63)
      REAL*4 RDATA(4200000)
      INTEGER*4 NN(3), ISIGN, NDIM
      REAL*8 IVAL

C     NSPHAN is the number of reciprocal lattice points that lie in
C     the spherical annulus corresponding to I-th point in the
C     direction (1,1,1) in Cartesian coordinates. 
C       INTEGER*4 NSPHAN(NL)
      REAL*8 D(0:3, NL), DIF(0:3,NL),SUP(0:3,NL),SUP2(0:3,NL)
      REAL*8 DA(0:3,NL),SA(0:3,NL),S2A(0:3,NL)
      INTEGER*4 KLIMD, NIVALD
      INTEGER*4 KLIMS, NIVALS
      INTEGER*4 KLIMS2, NIVALS2
      INTEGER*4 NFIL, NUMFIL

      REAL*8 VFRAC, SCTAR, SCTBR, BGRUND(NL)
      REAL*8 JR  
      INTEGER JN,ipoint
      CHARACTER*80 JCHAR
      INTEGER*4 I1, I2, I3, N1, N2, N3, M1, M2, M3
      INTEGER*4 N, I, JJ, IDATOMI, NL1
      INTEGER*4 NCONF, NCONINI, NCONFIN,NCUT,AA,av

      REAL*8 AVETA,r1,r2,r11,r12,r13
      REAL*8 KINT(0:3), DELK, KSCAL, ISCAL
      REAL*8 MOM(0:2,0:3),MOM1(0:2,0:3)
      REAL*8 TIME
      REAL*8 dosize,inten,intenper, degorder
      REAL*8 DBKG, SBKG, TMP

      INTEGER*4 KMCIN, KMCOUT, KMCOUTS,KMCRA, KMCDOS
      INTEGER*4 KMCOUTD , KMCOUTS2,KMCOUTS2A,KMCOUTDA
      CHARACTER*20 FNAMIN, FOUT, FOUTS, FARICH, FBRICH 
      CHARACTER*20 FOUTD, FOUTS2,FOUTDA,FOUTS2A,FOUTDOS
      DATA KMCIN, KMCOUT, KMCOUTS2,KMCOUTS2A /21, 22, 23, 24/
      DATA KMCOUTD, KMCOUTS, KMCOUTDA,KMCRA,KMCDOS /25, 26, 27, 28,29/
      DATA FNAMIN /'difsupain'/
      DATA FOUT, FOUTD, FOUTS /'KMC.tdif','KMC.ddif','KMC.sdif'/
      DATA FOUTS2,FOUTDA,FOUTS2A /'KMC.rdif','KMC.dadif','KMC.sadif'/
      DATA FOUTDOS /'KMC.dos'/

C     Input section
      WRITE (*,*) 'Start calculating sf for IT',IT
      OPEN (KMCIN,FILE=FNAMIN,STATUS='OLD')
      READ (KMCIN,*) 
      READ (KMCIN,*)
      READ (KMCIN,*)
      READ (KMCIN,*)
      READ (KMCIN,*)
      READ (KMCIN,*)
      OPEN (UNIT=KMCOUT, FILE=FOUT, ACCESS='APPEND')
      OPEN (UNIT=KMCOUTD, FILE=FOUTD, ACCESS='APPEND')
      OPEN (UNIT=KMCOUTS, FILE=FOUTS, ACCESS='APPEND')
      OPEN (UNIT=KMCOUTS2, FILE=FOUTS2, ACCESS='APPEND')
      OPEN (UNIT=KMCOUTDA, FILE=FOUTDA, ACCESS='APPEND')
      OPEN (UNIT=KMCRA, FILE='KMC.ra', ACCESS='APPEND')
      OPEN (UNIT=KMCOUTS2A, FILE=FOUTS2A, ACCESS='APPEND')
      OPEN (UNIT=KMCDOS, FILE=FOUTDOS, ACCESS='APPEND')
      OPEN (UNIT=30, FILE='KMCs0.dat', ACCESS='APPEND')

      NUMFIL=1
      NCONINI=1
      NCONFIN=1
      NCUT=1
      AV=1

      READ (KMCIN, *)
      READ (KMCIN, *)
      READ (KMCIN, *)
      READ (KMCIN, *)
      READ (KMCIN, *)

      READ (KMCIN, *) KLIMD  
      READ (KMCIN, *) KLIMS, KLIMS2
      CLOSE (KMCIN)

C     END OF INPUT SECTION

      NLCB = NL*NL*NL
      NLCB2 = NLCB*2

      IF (NL.EQ.64) THEN
c        For L12
         intenper = 16384.0
c        For L10
c         intenper = 21845.33333
      ELSE
c        For L12
         Intenper = 131072.0
c        For L10
c         Intenper = 174762.66667
      END IF

      DO 110 N = 1, NL    
         BGRUND(N) = 0.0
 110  CONTINUE

      DO N=1,NL
         DA(0,N) = 0.0
         DA(1,N) = 0.0
         DA(2,N) = 0.0
         DA(3,N) = 0.0
         SA(0,N) = 0.0
         SA(1,N) = 0.0
         SA(2,N) = 0.0
         SA(3,N) = 0.0
         S2A(0,N) = 0.0
         S2A(1,N) = 0.0
         S2A(2,N) = 0.0
         S2A(3,N) = 0.0
      END DO

      r1=0.
      r2=0.
      r11=0.
      r12=0.
      r13=0.
      ipoint=0 
      AA=0

C     The out loop        
      DO 2000 NCONF = NCONINI, NCONFIN
         WRITE (KMCOUT,*)
         WRITE (KMCOUTD,*)
         WRITE (KMCOUTS,*)
         WRITE (KMCOUTS2,*)

         DO 120 N = 1, NL
            DIF(0,N) = 0.0
            DIF(1,N) = 0.0
            DIF(2,N) = 0.0
            DIF(3,N) = 0.0
            SUP(0,N) = 0.0
            SUP(1,N) = 0.0
            SUP(2,N) = 0.0
            SUP(3,N) = 0.0
            SUP2(0,N) = 0.0
            SUP2(1,N) = 0.0
            SUP2(2,N) = 0.0
            SUP2(3,N) = 0.0
 120     CONTINUE

         DO 1000 NFIL = 1, NUMFIL 
C     AVETA is the average value of ETA which takes the value of 1 (or -1)
C     if the site is occupied by a B (or an A) atom. Thus ETA is exactly
C     the same as our variable ID (check the constants AID, BID, and VID
C     at the beginning of the program).
            AVETA = 0.0
            DO 250 I3 = 1, NL
            DO 250 I2 = 1, NL
            DO 250 I1 = 1, NL
               IDATOMI = ID(I1,I2,I3)
               AVETA = AVETA + REAL(IDATOMI)
 250        CONTINUE
            AVETA = AVETA / REAL(NLCB)

C     Generate the complex array describing the configuration, and
C     put it in a form suitable for calling the FFT subroutine:
C     The RDATA array is a real array of length 2*NLCB.
            JJ = 1
            DO 400 I3 = 1, NL
            DO 400 I2 = 1, NL
            DO 400 I1 = 1, NL
               IDATOMI = ID(I1,I2,I3)
               RDATA(JJ) = - CONCB
               IF (IDATOMI.EQ.BID) RDATA(JJ) = 1.0 - CONCB
               RDATA(JJ+1) = 0.0
               JJ = JJ + 2
 400        CONTINUE

            ISIGN = 1
            NDIM = 3
            NN(1) = NL
            NN(2) = NL
            NN(3) = NL
            CALL FOURN(RDATA, NN, NDIM, ISIGN, NLCB2)

C     From RDATA, which is a real array of twice the length of the
C     corresponding complex array, calculate the intensity of the
C     spots in the reciprocal lattice.
            JJ = 1
            DO 500 I3 = 0, NL-1
            DO 500 I2 = 0, NL-1
            DO 500 I1 = 0, NL-1
               S1(I1,I2,I3) = RDATA(JJ)**2 + RDATA(JJ+1)**2
               JJ = JJ + 2
 500        CONTINUE 
C           print *, 's1(0,0,0)=', s1(0,0,0)

            IVAL = SQRT(3.0) 
            NIVALD = KLIMD
            CALL DSCTR (NL, S1, D, IVAL, NIVALD, KLIMD, DBKG) 

            DO 700 N = 1, NL
               DIF(0,N) = DIF(0,N) + D(0,N) / REAL(NUMFIL)
               DIF(1,N) = DIF(1,N) + D(1,N) / REAL(NUMFIL)
               DIF(2,N) = DIF(2,N) + D(2,N) / REAL(NUMFIL)
               DIF(3,N) = DIF(3,N) + D(3,N) / REAL(NUMFIL)
 700        CONTINUE

            NIVALS = KLIMS
            CALL DSCTRS (NL, S1, D, IVAL, NIVALS, KLIMS, INTEN, SBKG)

            DO 800 N = 1, NL
               SUP(0,N) = SUP(0,N) + D(0,N) / REAL(NUMFIL)
               SUP(1,N) = SUP(1,N) + D(1,N) / REAL(NUMFIL)
               SUP(2,N) = SUP(2,N) + D(2,N) / REAL(NUMFIL)
               SUP(3,N) = SUP(3,N) + D(3,N) / REAL(NUMFIL)
 800        CONTINUE

            NIVALS2 = KLIMS2
            CALL DSCTRS2 (NL, S1, D, IVAL, NIVALS2, KLIMS2)

            DO 900 N = 1, NL
               SUP2(0,N) = SUP2(0,N) + D(0,N) / REAL(NUMFIL)
               SUP2(1,N) = SUP2(1,N) + D(1,N) / REAL(NUMFIL)
               SUP2(2,N) = SUP2(2,N) + D(2,N) / REAL(NUMFIL)
               SUP2(3,N) = SUP2(3,N) + D(3,N) / REAL(NUMFIL)
 900        CONTINUE
 1000    CONTINUE

         KINT(0) = 2.0 * PI * SQRT(3.0) / REAL(NL)
         KINT(1) = 4.0 * PI / REAL(NL)
         KINT(2) = 4.0 * PI * SQRT(2.0) / REAL(NL)
         KINT(3) = 2.0 * PI * SQRT(3.0) / REAL(NL)
         WRITE (KMCOUT, '(I5, A, E14.7)') IT, ' ', TIME

         DO 1025 N = 1, KLIMD
            N1 = N - 1
            WRITE(KMCOUT,'(I4,A,F8.4,2(A,F9.4),A,F8.4,A,F9.4,A,F8.4,A,
     /         F9.4)')
     /          N1, ' ', N1*KINT(0), ' ', DIF(0,N), ' ', DIF(3,N), 
     /              ' ', N1*KINT(1), ' ', DIF(1,N),  
     /              ' ', N1*KINT(2), ' ', DIF(2,N)        
 1025    CONTINUE

         DO 1050 N = 1, NL
            DIF(0,N) = DIF(0,N) - BGRUND(N)
 1050    CONTINUE 

         DO 1100 N = 0,2
         DO 1100 I = 0,3
            MOM(N,I) = 0.0
            MOM1(N,I) = 0.0
1100     CONTINUE

         DO 1200 N1 = 1, KLIMS2
            N = N1 - 1
            TMP = SUP(0,N1) - SBKG
            IF (TMP .LT. 0) TMP = 0
            MOM1(0,0) = MOM1(0,0) + TMP
            MOM1(1,0) = MOM1(1,0) + TMP * N
            MOM1(2,0) = MOM1(2,0) + TMP * N * N

            DO 1190 I = 1, 3
               MOM1(0,I) = MOM1(0,I) + SUP2(I,N1)
               MOM1(1,I) = MOM1(1,I) + SUP2(I,N1) * N
               MOM1(2,I) = MOM1(2,I) + SUP2(I,N1) * N * N
 1190       CONTINUE
 1200    CONTINUE

         DO 1201 N1 = 1, KLIMD
            N = N1 - 1
            DO 1191 I = 0, 3
               MOM(0,I) = MOM(0,I) + DIF(I,N1)
               MOM(1,I) = MOM(1,I) + DIF(I,N1) * N
               MOM(2,I) = MOM(2,I) + DIF(I,N1) * N * N
 1191       CONTINUE
 1201    CONTINUE

         DO 1250 I = 0, 3
            MOM(1,I) = MOM(1,I) * KINT(I) / MOM(0,I)
            MOM(2,I) = MOM(2,I) * KINT(I)**2 / MOM(0,I)

            MOM1(1,I) = MOM1(1,I) * (KINT(0)) / MOM1(0,I)
            MOM1(2,I) = MOM1(2,I) * (KINT(0))**2 / MOM1(0,I)
 1250    CONTINUE

         dosize = 4.0/sqrt(MOM1(2,0))
         degorder = sqrt(inten/intenper)
	   WRITE (*,'(A,F6.4)') ' DEGREE OF ORDER: ',degorder
         WRITE (KMCDOS,'(E12.3,2F12.5)') TIME,dosize,degorder

         ipoint=ipoint+1
         r1=r1+(MOM1(1,0))
         r2=r2+sqrt(MOM1(2,0))

c         r11=r11+sqrt(MOM1(2,1))

         r12=r12+MOM(1,0)
C         r13=r13+sqrt(MOM(2,0))

         if (mod(ipoint,av).eq.0) then
            ipoint=0
            r1=r1/av
            r2=r2/av
            r11=r11/av
            r12=r12/av
            r13=r13/av
            WRITE (KMCRA, '(E14.7, 5(A,F10.5))')     TIME,
     /                        ' ', 1./r12,
     /                        ' ', 1./r13,
     /                        ' ', 1./r11,
     /                        ' ', 1./r1,
     /                        ' ', 1./r2
            r1=0. 
            r2=0.
            r11=0.
            r12=0.
            r13=0.
         end if

         WRITE (KMCOUTD, '(A,I5, A, E14.7, 3(A,F10.5))') '#',IT,
     /                        ' ', TIME,
     /                        ' ', MOM(1,0), 
     /                        ' ', MOM(2,0) 

         WRITE (KMCOUTS, '(A, I5, A, E14.7, A, F10.5)') '#', 
     /                 IT, ' ', TIME, ' ', MOM(1,0) 
         WRITE (KMCOUTS2, '(A,I5, A, E14.7, A, F10.5)') '#', 
     /                 IT, ' ', TIME, ' ', MOM(1,0) 
         DO 1300 N1 = 1, NL
            N = N1 - 1
            DELK = N * KINT(3)
            KSCAL = DELK / MOM(1,0)

            IF (N1.LE.KLIMD) THEN
               ISCAL = DIF(0,N1) * MOM(1,0)**3  
               WRITE (KMCOUTD, '(I5, 4(A, F10.5))') N, ' ', 
     /                                   DELK, ' ', DIF(0,N1), ' ',
     /                                   KSCAL, ' ', ISCAL  
            END IF

            IF (N1.LE.KLIMS) THEN
               ISCAL = SUP(0,N1) * MOM(1,0)**3  
               WRITE (KMCOUTS, '(I5, 4(A, D13.5))') N, ' ', 
     /                   DELK, ' ', SUP(0,N1), ' ',
     /                                   KSCAL, ' ', ISCAL  
            END IF

            IF (N1.LE.KLIMS2) THEN
               WRITE (KMCOUTS2, '(I5, 4(A, D13.5))') N, ' ', 
     /                   DELK, ' ', SUP2(1,N1), ' ',
     /                   SUP2(2,N1), ' ', SUP2(3,N1)  
            END IF

 1300    CONTINUE 
             
         AA=AA+1
         DO NL1 = 1, NL
            DA(0,NL1)=DA(0,NL1)+DIF(0,NL1)
            DA(1,NL1)=DA(1,NL1)+DIF(1,NL1)
            DA(2,NL1)=DA(2,NL1)+DIF(2,NL1)
            DA(3,NL1)=DA(3,NL1)+DIF(3,NL1)
               
            SA(0,NL1)=SA(0,NL1)+SUP(0,NL1)
            SA(1,NL1)=SA(1,NL1)+SUP(1,NL1)
            SA(2,NL1)=SA(2,NL1)+SUP(2,NL1)
            SA(3,NL1)=SA(3,NL1)+SUP(3,NL1)
               
            S2A(0,NL1)=S2A(0,NL1)+SUP2(0,NL1)
            S2A(1,NL1)=S2A(1,NL1)+SUP2(1,NL1)
            S2A(2,NL1)=S2A(2,NL1)+SUP2(2,NL1)
            S2A(3,NL1)=S2A(3,NL1)+SUP2(3,NL1)
         END DO

         if (mod(NCONF-1,NCUT).eq.0) then
            WRITE (KMCOUTDA, '(A,I5)') '#',IT
            WRITE (KMCOUTS2A, '(A,I5)') '#', IT
            WRITE (30,'(I5,A,2D14.5)') IT, '  ', SA(0,1)/AA,DA(0,2)/AA

            DO N1 = 1, NL
               N = N1 - 1
               DELK = N * KINT(3)
               KSCAL = DELK / MOM(1,0)

               IF (N1.LE.KLIMD) THEN
                  ISCAL = DIF(0,N1) * MOM(1,0)**3 
                  WRITE (KMCOUTDA, '(I5, 4(A, F10.5))') N, ' ', 
     /                       DELK, ' ', DA(0,N1)/AA
               END IF

               IF (N1.LE.KLIMS2) THEN
                  WRITE (KMCOUTS2A, '(I5, 5(A, D10.5))') N, ' ',
     /               DELK, ' ', S2A(1,N1)/AA, ' ',
     /               S2A(2,N1)/AA, ' ', S2A(3,N1)/AA, ' ', SA(0,N1)/AA
               END IF

               DA(0,N1) = 0.0
               DA(1,N1) = 0.0
               DA(2,N1) = 0.0
               DA(3,N1) = 0.0
               SA(0,N1) = 0.0
               SA(1,N1) = 0.0
               SA(2,N1) = 0.0
               SA(3,N1) = 0.0
               S2A(0,N1) = 0.0
               S2A(1,N1) = 0.0
               S2A(2,N1) = 0.0
               S2A(3,N1) = 0.0
            END DO

            AA=0
         end if
 2000 continue

      CLOSE (KMCRA)
      CLOSE (KMCOUT)
      CLOSE (KMCOUTD) 
      CLOSE (KMCOUTS) 
      CLOSE (KMCOUTS2) 
      CLOSE (KMCOUTDA) 
      CLOSE (KMCOUTS2A) 
      CLOSE (KMCDOS)
      CLOSE (30)

      RETURN
      END

CCCCCCC*************************************************************
      SUBROUTINE DSCTR (NL, S, D, IVAL, NIVAL, KLIM, BKG)

      IMPLICIT NONE
       
      INTEGER*4 NL, KLIM, NIVAL
      REAL*4 S(0:NL-1,0:NL-1,0:NL-1)
      REAL*8 D(0:3,NL)
      REAL*8 IVAL
      REAL*8 NLCB, NLCB3, NLCB4, NLCB6
      INTEGER N, I, NM, N1, N2, N3, I1, I2, I3, K1, K2, K3, KBASE
      REAL*8 KMX, MAGTDE, BKG
      INTEGER SUBL, NMX, NDELK, NBKG
      INTEGER NSPHAN(NL)

      DO 100 N = 1, NL
         D(0,N) = 0.0
         D(1,N) = 0.0
         D(2,N) = 0.0
         D(3,N) = 0.0
         NSPHAN(N) = 0
 100  CONTINUE

      NLCB = NL*NL*NL
      NLCB3 = 3.0 * NLCB     
      NLCB4 = 4.0 * NLCB
      NLCB6 = 6.0 * NLCB

      DO 200 N = 0, KLIM - 1
         I = N + 1
         N2 = 2*N 
         NM = NL - N 
         IF (NM.GE.NL) NM = NM - NL
         IF (N2.GE.NL) N2=N2-NL
         D(1,I) = D(1,I) + (S(0,N,N) + S(N,N,0) + S(N,0,N))
     /                     / NLCB3
         D(3,I) = D(3,I) + (S(N,0,0) + S(0,N,0) + S(0,0,N)
     /                      + S(N,N,N)) / NLCB4
         D(2,I) = D(2,I) + (S(N,N,N2) + S(N,N2,N) + S(N2,N,N)
     /                   + S(0,N,NM) + S(N,0,NM) + S(N,NM,0)) / NLCB6
 200  CONTINUE

      KMX = NIVAL * IVAL
      KBASE = 10 * NL 
      NMX = 2 + INT(KMX) / 2 
      DO 300 I1 = -NMX, NMX
      DO 300 I2 = -NMX, NMX
      DO 300 I3 = -NMX, NMX
         DO 290 SUBL = 1, 2
            IF (SUBL.EQ.1) THEN
               N1 = 2 * I1
               N2 = 2 * I2
               N3 = 2 * I3
            ELSE         
               N1 = 1 + 2 * I1
               N2 = 1 + 2 * I2
               N3 = 1 + 2 * I3
            END IF
            MAGTDE = REAL(N1*N1) + REAL(N2*N2) + REAL(N3*N3)
            MAGTDE = SQRT(MAGTDE)
            IF (MAGTDE.LE.KMX) THEN
               K1 = MOD((KBASE + (N1 + N2) / 2),NL)
               K2 = MOD((KBASE + (N2 + N3) / 2),NL)
               K3 = MOD((KBASE + (N1 + N3) / 2),NL)
               NDELK = 1 + NINT(MAGTDE/IVAL)
               NSPHAN(NDELK) = NSPHAN(NDELK) + 1
               D(0,NDELK) = D(0,NDELK) + S(K1,K2,K3)
            END IF
 290     CONTINUE
 300  CONTINUE
     
      DO 400 N = 1, NL
         IF (NSPHAN(N).NE.0) THEN
            D(0,N) = D(0,N) / (REAL(NLCB) * NSPHAN(N))
         END IF
 400  CONTINUE

      BKG = 0
      NBKG=NIVAL/4
      DO 500 N = NIVAL-NBKG, NIVAL
	   BKG = BKG+D(0,N)
 500  CONTINUE
      BKG = BKG/REAL(NBKG+1)

      RETURN
      END

CCCCCCC*************************************************************
      SUBROUTINE DSCTRS (NL, S, D, IVAL, NIVAL, KLIM, INTEN, BKG)

C    Average diffuse scattering intensity around the superlattice
C    reflections at 100, 010 and 001 for a fcc alloy to detect
C    the presence of L12 and/or L10 ordered phases. 


      IMPLICIT NONE
      INTEGER*4 NL, KLIM, NIVAL
      REAL*4 S(0:NL-1,0:NL-1,0:NL-1)
      REAL*8 D(0:3,NL)
      REAL*8 IVAL
      REAL*8 NLCB, NLCB3, NLCB4, NLCB6
      INTEGER N, I, NM, N1, N2, N3, I1, I2, I3, K1, K2, K3, KBASE
      INTEGER NH, NLH, ISUP
      REAL*8 KMX, KMX2, MAGTDE, INTEN, BKG
      INTEGER SUBL, NMX, NDELK,NBKG
      INTEGER NSPHAN(NL)

      DO 100 N = 1, NL
         D(0,N) = 0.0
         D(1,N) = 0.0
         D(2,N) = 0.0
         D(3,N) = 0.0
         NSPHAN(N) = 0
 100  CONTINUE
      NLCB = NL*NL*NL
      INTEN = 0.
      NLH = NL / 2  
      KMX = NIVAL * IVAL
      KMX2 = KMX / 5.0
      KBASE = 10 * NL 
      NMX = 2 + INT(KMX) / 2 
      DO 300 I1 = -NMX, NMX
      DO 300 I2 = -NMX, NMX
      DO 300 I3 = -NMX, NMX
         DO 290 SUBL = 1, 2
            IF (SUBL.EQ.1) THEN
               N1 = 2 * I1
               N2 = 2 * I2
               N3 = 2 * I3
            ELSE         
               N1 = 1 + 2 * I1
               N2 = 1 + 2 * I2
               N3 = 1 + 2 * I3
            END IF
            MAGTDE = REAL(N1*N1) + REAL(N2*N2) + REAL(N3*N3)
            MAGTDE = SQRT(MAGTDE)
            IF (MAGTDE.LE.KMX) THEN
               DO 280 ISUP = 1, 3
                  IF (ISUP.EQ.1) THEN
                     K1 = MOD((KBASE + NLH + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + NLH + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + (N1 + N3) / 2),NL)
                  END IF 
                  IF (ISUP.EQ.2) THEN
                     K1 = MOD((KBASE + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + NLH + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + NLH + (N1 + N3) / 2),NL)
                  END IF 
                  IF (ISUP.EQ.3) THEN
                     K1 = MOD((KBASE + NLH + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + NLH + (N1 + N3) / 2),NL)
                  END IF 
                  NDELK = 1 + NINT(MAGTDE/IVAL)
                  NSPHAN(NDELK) = NSPHAN(NDELK) + 1
                  D(0,NDELK) = D(0,NDELK) + S(K1,K2,K3)
C                  IF (MAGTDE.LE.KMX2) THEN
C                     INTEN = INTEN + S(K1,K2,K3)
C                  END IF
 280           CONTINUE
            END IF
 290     CONTINUE
 300  CONTINUE

      DO 400 N = 1, NL
         IF (NSPHAN(N).NE.0) THEN
            D(0,N) = D(0,N) / (REAL(NLCB) * NSPHAN(N))
         END IF
 400  CONTINUE
      
      DO N=1,NIVAL
         INTEN=INTEN+D(0,N)
      END DO

      BKG = 0
      NBKG=NIVAL/4
      DO 500 N = NIVAL-NBKG, NIVAL
         BKG = BKG+D(0,N)
 500  CONTINUE
      BKG = BKG/REAL(NBKG+1)
      INTEN = ABS(INTEN-NIVAL*BKG) 

      RETURN
      END


CCCCCCC*************************************************************
      SUBROUTINE DSCTRS2 (NL, S, D, IVAL, NIVAL, KLIM)

C    Individual diffuse scattering intensity around the superlattice
C    reflections 100, 010 and 001 to detect the formation of a L10 phase

      IMPLICIT NONE
       
      INTEGER*4 NL, KLIM, NIVAL
      REAL*4 S(0:NL-1,0:NL-1,0:NL-1)
      REAL*8 D(0:3,NL)
      REAL*8 IVAL
      REAL*8 NLCB, NLCB3, NLCB4, NLCB6
      INTEGER N, I, NM, N1, N2, N3, I1, I2, I3, K1, K2, K3, KBASE
      INTEGER NH, NLH, ISUP
      REAL*8 KMX, MAGTDE
      INTEGER SUBL, NMX, NDELK
      INTEGER NSPHAN(NL)

      DO 100 N = 1, NL
         D(0,N) = 0.0
         D(1,N) = 0.0
         D(2,N) = 0.0
         D(3,N) = 0.0
         NSPHAN(N) = 0
 100  CONTINUE
      NLCB = NL*NL*NL
      NLH = NL / 2  
      KMX = NIVAL * IVAL
      KBASE = 10 * NL 
      NMX = 2 + INT(KMX) / 2 
      DO 300 I1 = -NMX, NMX
      DO 300 I2 = -NMX, NMX
      DO 300 I3 = -NMX, NMX
         DO 290 SUBL = 1, 2
            IF (SUBL.EQ.1) THEN
               N1 = 2 * I1
               N2 = 2 * I2
               N3 = 2 * I3
            ELSE         
               N1 = 1 + 2 * I1
               N2 = 1 + 2 * I2
               N3 = 1 + 2 * I3
            END IF
            MAGTDE = REAL(N1*N1) + REAL(N2*N2) + REAL(N3*N3)
            MAGTDE = SQRT(MAGTDE)
            IF (MAGTDE.LE.KMX) THEN
               DO 280 ISUP = 1, 3
                  IF (ISUP.EQ.1) THEN
                     K1 = MOD((KBASE + NLH + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + NLH + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + (N1 + N3) / 2),NL)
                  END IF 
                  IF (ISUP.EQ.2) THEN
                     K1 = MOD((KBASE + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + NLH + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + NLH + (N1 + N3) / 2),NL)
                  END IF 
                  IF (ISUP.EQ.3) THEN
                     K1 = MOD((KBASE + NLH + (N1 + N2) / 2),NL)
                     K2 = MOD((KBASE + (N2 + N3) / 2),NL)
                     K3 = MOD((KBASE + NLH + (N1 + N3) / 2),NL)
                  END IF 

                  NDELK = 1 + NINT(MAGTDE/IVAL)
                  NSPHAN(NDELK) = NSPHAN(NDELK) + 1
                  D(ISUP,NDELK) = D(ISUP,NDELK) + S(K1,K2,K3)
 280           CONTINUE
            END IF
 290     CONTINUE
 300  CONTINUE

      DO 400 N = 1, NL
         IF (NSPHAN(N).NE.0) THEN
            D(1,N) = 3 * D(1,N) / (REAL(NLCB) * NSPHAN(N))
            D(2,N) = 3 * D(2,N) / (REAL(NLCB) * NSPHAN(N))
            D(3,N) = 3 * D(3,N) / (REAL(NLCB) * NSPHAN(N))
         END IF
 400  CONTINUE

      RETURN
      END

