      subroutine vcnfini2 (ID1D, NLCB, NBTOT, NVTOT, LUCONIN, FCONIN,
     /              IRAN, AID, BID, VID, NL1, NL2, TIME, LOCV, NVMX)
 
C  Specific for pairr.f : non-regular rhomboedron crystal L1xL1xL2
C If NBTOT is positive, and greater than five
C  it represents the total number of B atoms
C  in the crystal and the initial configuration is determined randomly.
c If NBTOT equals one, build a slice of pure B in pure A ; 
c  the average composition is then 0.5/0.5.
c If NBTOT equals two, build a small rhomboedron of pure B in pure A
c  the average composition is then 1/8
c If NBTOT equals four, build a multilayer of 4 A/B sequences
c  the average composiiton is then 0.5
C If NBTOT is negative or zero, 
C  the configuration is read from an input file
C  after skipping (-NBTOT) configurations in it.
C ID1D is the one dimensional array that stores the configuration.
C NLCB is the cube of linear dimension of the crystal.
C IRAN is the seed for the random number generator.
C LUCONIN is the unit number of the conf. input file.
 
       INTEGER*4 NLCB, LUCONIN, IRAN, NBTOT, NVTOT, NL1, NL2, NVMX
       INTEGER*4 LOCV(3, NVMX)
       INTEGER*2 AID, BID, VID
       INTEGER*2 ID1D(NLCB)
       CHARACTER*20 FCONIN
       REAL*8 TIME
 
       INTEGER BUFSIZ,WORDLEN,NBUF
       PARAMETER (BUFSIZ=256,WORDLEN=32)
 
       INTEGER BUFFER(BUFSIZ)
       INTEGER N, NSKIP, WORD, BIT, BUF, X, BITPOS
       INTEGER BUFCOMP, WORDCOMP
       INTEGER NVAC, VACLOC, I1, I2, I3, AA, AA2, BB, BB2, AMPL
       REAL*8 RANNO,RI1,RI2,XX,RNL, Pi
       DATA Pi /3.141592653589 /
 
C Initialize.
       DO 150 X = 1, NLCB
 150	  ID1D(X) = AID
 
      DO 160 N = 1, NVMX
       DO 160 I1 = 1, 3
          LOCV(I1,N) = 0
160    CONTINUE

       IF (NBTOT.GT.5) THEN
          WRITE (*,*) 'RANDOM CONFIGURATION. ', NBTOT, NVTOT
C Random configuration.
          TIME = 0.0
	  DO 200 N = 1, NBTOT + NVTOT
 199	     X = 1 + INT(RANNO(IRAN) * NLCB)
             IF (ID1D(X).NE.AID) GOTO 199
             IF (N.LE.NBTOT) THEN
                ID1D(X) = BID
             ELSE
                ID1D(X) = VID
                I3 = (X - 1) / (NL1 * NL1)
                I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
                I1 = X - I3 * NL1 * NL1 - I2 * NL1
                LOCV(3, N - NBTOT) = I3 + 1
                LOCV(2, N - NBTOT) = I2 + 1
                LOCV(1, N - NBTOT) = I1
             END IF
 200      CONTINUE
 
       ELSE IF (NBTOT.EQ.1) THEN
          WRITE (*,*) 'SLICE OF PURE B . '
c  Slice of pure B ; vacancies are put in pure A.	
          TIME = 0.0
	  DO 250 X = NLCB/4 + 1 , (3*NLCB)/4
	    ID1D(X) = BID
 250	  CONTINUE
	  DO 260 N = 1, NVTOT
 259 	    X = 1 + INT(RANNO(IRAN) * NLCB)
	    IF (ID1D(X).NE.AID) GOTO 259
            ID1D(X) = VID
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
            LOCV(3, N) = I3 + 1
            LOCV(2, N) = I2 + 1
            LOCV(1, N) = I1
 260      CONTINUE
	  NBTOT = NLCB / 2 

       ELSE IF (NBTOT.EQ.2) THEN
          WRITE (*,*) 'PRECIPITATE OF PURE B . '
c  precipitate of pure B ; vacancies are put in pure A.	
          TIME = 0.0
	  AA = NL1 / 4
	  AA2 = NL2 /4
	  BB = 3 * AA
	  BB2 = 3 * AA2
	  DO 270 X = 1 , NLCB
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
	    I3 = I3 + 1
	    I2 = I2 + 1
	    IF (I1.GT.AA.AND.I1.LE.BB.AND.I2.GT.AA.AND.I2.LE.BB.
     /          AND.I3.GT.AA2.AND.I3.LE.BB2)  ID1D(X) = BID
 270	  CONTINUE
	  DO 280 N = 1, NVTOT
 279 	    X = 1 + INT(RANNO(IRAN) * NLCB)
	    IF (ID1D(X).NE.AID) GOTO 279
            ID1D(X) = VID
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
            LOCV(3, N) = I3 + 1
            LOCV(2, N) = I2 + 1
            LOCV(1, N) = I1
 280      CONTINUE
	  NBTOT = NLCB / 2 
 
       ELSE IF (NBTOT.EQ.3) THEN
          WRITE (*,*) '2 layers of A/B . '
c  2 Slices of pure B ; vacancies are put in pure A.	
          TIME = 0.0
	  DO 305 N = 1, 2
	    DO 310 X = (NLCB*(4*N-3))/8 + 1 , (NLCB*(4*N-1))/8
	      ID1D(X) = BID
 310	    CONTINUE
 305	  CONTINUE
	  DO 330 N = 1, NVTOT
 333 	    X = 1 + INT(RANNO(IRAN) * NLCB)
	    IF (ID1D(X).NE.AID) GOTO 333
            ID1D(X) = VID
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
            LOCV(3, N) = I3 + 1
            LOCV(2, N) = I2 + 1
            LOCV(1, N) = I1
 330      CONTINUE
	  NBTOT = NLCB / 2 

       ELSE IF (NBTOT.EQ.4) THEN
          WRITE (*,*) 'Multilayer A/B . '
c  4 Slices of pure B ; vacancies are put in pure A.	
          TIME = 0.0
	  DO 500 N = 1, 4
	    DO 510 X = (NLCB*(2*N-1))/8 + 1 , (N*NLCB)/4
	      ID1D(X) = BID
 510	    CONTINUE
 500	  CONTINUE
	  DO 600 N = 1, NVTOT
 599 	    X = 1 + INT(RANNO(IRAN) * NLCB)
	    IF (ID1D(X).NE.AID) GOTO 599
            ID1D(X) = VID
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
            LOCV(3, N) = I3 + 1
            LOCV(2, N) = I2 + 1
            LOCV(1, N) = I1
 600      CONTINUE
	  NBTOT = NLCB / 2 

       ELSE IF (NBTOT.EQ.5) THEN
          WRITE (*,*) 'SINE WAVE OF PURE B . '
c  Wave of pure B ; vacancies are put in pure A.	
          TIME = 0.0
	  AMPL = 10
	  DO 700 X = NLCB/4+AMPL*NL1*NL1+1,(3*NLCB)/4-AMPL*NL1*NL1
	    ID1D(X) = BID
 700	  CONTINUE
	  DO 710 X=NLCB/4+1-AMPL*NL1*NL1,NLCB/4+AMPL*NL1*NL1
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
	    I3 = I3 + 1
	    I2 = I2 + 1
	    RI2 = REAL(I2) + REAL(I3)/3. 
	    RI1 = REAL(I1) + REAL(I3)/3.
	    RNL = REAL(NL1)
	    XX = REAL(AMPL)*SIN(2.*Pi*RI1/RNL)*SIN(2.*Pi*RI2/RNL)
	    AA = NL2 / 4 + NINT(XX)
	    IF (I3.GT.AA)  ID1D(X) = BID
 710	  CONTINUE
	  DO 720 X=3*NLCB/4+1-AMPL*NL1*NL1,3*NLCB/4+AMPL*NL1*NL1
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
	    I3 = I3 + 1
	    I2 = I2 + 1
	    RI2 = REAL(I2) + REAL(I3)/3.  
	    RI1 = REAL(I1) + REAL(I3)/3.
	    RNL = REAL(NL1)
	    XX = REAL(AMPL)*SIN(2.*Pi*RI1/RNL)*SIN(2.*Pi*RI2/RNL)
	    AA = 3*NL2 / 4 + NINT(XX)
	    IF (I3.LT.AA)  ID1D(X) = BID
 720	  CONTINUE
	  DO 750 N = 1, NVTOT
 749 	    X = 1 + INT(RANNO(IRAN) * NLCB)
	    IF (ID1D(X).NE.AID) GOTO 749
            ID1D(X) = VID
            I3 = (X - 1) / (NL1 * NL1)
            I2 = ((X - I3 * NL1 * NL1) - 1) / NL1 
            I1 = X - I3 * NL1 * NL1 - I2 * NL1
            LOCV(3, N) = I3 + 1
            LOCV(2, N) = I2 + 1
            LOCV(1, N) = I1
 750      CONTINUE
	  NBTOT = NLCB / 2 

 
       ELSE
C Read the input configuration from a file.
c First skip N-1 conf's in this file.
         WRITE(*,*) 'READING CONFIGURATION FROM ', FCONIN
         OPEN(UNIT=LUCONIN,FILE=FCONIN,STATUS='OLD',FORM='UNFORMATTED')
          NBUF = 1 + (NLCB / (WORDLEN * BUFSIZ))
          NBAT=0
          NSKIP = - NBTOT
          DO 300 n = 1, NSKIP
             READ (LUCONIN,END=1111) TIME
             READ (LUCONIN,END=1111) BUFFER
             DO 290 BUF = 1,NBUF
                READ (LUCONIN,END=1111) BUFFER
 290         CONTINUE
 300      CONTINUE
 
          READ (LUCONIN,END=1111) TIME
          READ (LUCONIN,END=1111) BUFFER
          N = 1
          IRAN = BUFFER(N)
          N = 2
          NVTOT = BUFFER(N)
          DO 350 NVAC = 1, NVTOT
            I1 = BUFFER(N+1)
            I2 = BUFFER(N+2)
            I3 = BUFFER(N+3)
            VACLOC = (I3 - 1) * NL1 * NL1 + (I2 - 1) * NL1 + I1
            ID1D(VACLOC) = VID
            LOCV(1, NVAC) = I1
            LOCV(2, NVAC) = I2
            LOCV(3, NVAC) = I3
            IF ((VACLOC.LE.0).OR.(VACLOC.GE.NLCB)) WRITE (*,*) 'WHAT?'
            N = N+3
 350      CONTINUE
 
          DO 400 BUF = 1, NBUF
           READ (LUCONIN,END=1111) BUFFER
           BUFCOMP = (BUF - 1) * BUFSIZ * WORDLEN
           DO 390 WORD = 1, BUFSIZ
            WORDCOMP = (WORD - 1) * WORDLEN
            DO 380 BIT = 1, WORDLEN
             BITPOS = WORDLEN - BIT
             X = BUFCOMP + WORDCOMP + BIT
             IF ((X.LE.NLCB).AND.(BTEST(BUFFER(WORD), BITPOS))) THEN
                NBAT = NBAT + 1
                ID1D(x) = BID
             END IF
 380        CONTINUE
 390       CONTINUE
 400      CONTINUE
          CLOSE (UNIT = LUCONIN)
 
          NBTOT = NBAT
       END IF
 1111       WRITE (*,*) 'INITIAL CONFIGURATION DONE.'
 
       RETURN
       END


