CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C      KINETIC MONTE CARLO CODE IN FCC WITH UPTO 2 NEIGHBOR SHELLS
C
C      Program tiphirr7.f : monte-carlo simulations of a 3d fcc crystal
C        under irradiation, pair exchanges at once;
C        thermal diffusion done by jumps of an isolated vacancy.
C        Rhomboedral cell.
C
C      ATTENTION: ONLY *TWO* NEIGHBOR SHELLS.
C      ATTENTION: UP TO TEN VACANCIES
C      
C      Version 4.0
C      Diffusion coefficient is calculated. New variables 
C      ENTITY and MVENTY are used for this purpose.
C
C      Configuration at any instant is in ID(X,Y,Z); an equivalent
C      1-D array ID1D(N) is also used. The program also uses LOCV(N,K)
C      storing the K-th component of the location of the N-th vacancy.
C      BNEI(X,Y,Z,N) and VNEI(X,Y,Z,N) store the number of B and V
C      neighbors in the N-th shell of site (X,Y,Z). LOCV, BNEI and
C      VNEI are only auxiliary information.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 

      IMPLICIT NONE
 
      REAL*8 VERS
      PARAMETER(VERS=1.0)
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     NL1, NL2 = Linear dimensions of unit cell
C     NSHMX = The highest level of interaction shell.
C     NEISHMX = The highest possible number of neighbors in any of
C               these shells.
C     NBMX = Max. number of B atoms in the unit cell.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER*4 NL1, NL2, NSHMX, NEISHMX, NLCB, NVMX, FNEIMX
      PARAMETER (NL1 = 64, NL2 = 64)
      PARAMETER (NLCB = NL1*NL1*NL2)
      PARAMETER (FNEIMX = 12)
      PARAMETER (NSHMX = 4)
      PARAMETER (NEISHMX = 24)
      PARAMETER (NVMX = 10)
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     AID, BID and VID are the numbers used for identifying the
C     atoms at lattice sites.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER*2 AID, BID, VID
      PARAMETER (AID = -1, VID = 0, BID = 1)
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Our program needs these following configuration-defining
C         variables to be up-to-date.
C     ID(K1,K2,K3) = Identity of atom occupying the position
C         (K1, K2, K3). 1 if it is B; 0 if it is A; -1 if it is V.
C     BNEI(N, K1, K2, K3) keeps track of the number of B neighbors
C         in the N-th shell of position (K1, K2, K3).
C     LOCV(I,N) stores the I-th coordinate (position)
C         of the N-th vacancy.
C     NBTOT is the total no. of B atoms; NVTOT is for vacancies.
C     NATOT is for the A atoms.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER*2 ID(NL1,NL1,NL2)
      INTEGER*2 ID1D(NLCB)
      EQUIVALENCE (ID1D(1), ID(1,1,1))
      INTEGER*2 BNEI(NSHMX, NL1,NL1,NL2)
      INTEGER*4 LOCV(3, NVMX)
      INTEGER*4 NBTOT, NVTOT, NATOT, NBTOTIN
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     MODL(K1) helps find the location in the basic unit cell
C        which corresponds to those in the neighboring cells.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER*4 HIMAX1, HIMAX2, LOMIN1, LOMIN2
      PARAMETER (HIMAX1 = 5*NL1 + 1, HIMAX2 = 5*NL2 + 1)
      PARAMETER (LOMIN1 = -5*NL1, LOMIN2 = -5*NL2)
      INTEGER*4 MODL1(LOMIN1:HIMAX1), MODL2(LOMIN2:HIMAX2)
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     NM(N) stores the total number of neighbor sites in N-th shell.
C     DELTA1N(N) is used to account for non-destroyal of the bond of
C        the species being exchanged.
C     ISHM is the actual level of interactions used ( < NSHMX).
C     WAB(N) the interaction energy for N-th shell.
C       (WAB = 2*EAB - EAA - EBB). Similar definition for WBV and WAV.
C     ALPHA = EAV - EAA; ALPHV = EAV - EVV.
C     EXALPHA(DIFF, N) = EXP(DIFF * ALPHA(N) / kT).
C     Similar definition for EXALPHV.
C     EXAB(DIFF,N) stores the probability (multiplicative)
C       contribution from the N-th shell.
C     SCEXENAB(DIFF, N) stores SCaled EXchange ENergy (additive)
C       contribution from the N-th shell.
C     NVEC(NSH,NEI,I) stores the I-th coordinate of the neighbor NEI
C       in the shell NSH.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      INTEGER*4 NM(NSHMX), DELTA1N(NSHMX), ISHM
      REAL*8 ENCOHNA, ENCOHNB, ENVFORA, ENVFORB
      real*8 NU, ENMV
      REAL*8 ENBASE,ENN1,ETOT,ENSAD
      REAL*8 WAB(NSHMX), WBV(NSHMX), WAV(NSHMX)
      REAL*8 EAA(NSHMX), EBB(NSHMX), EAB(NSHMX)
      REAL*8 EAV(NSHMX), EBV(NSHMX)
      INTEGER*4 NVEC(3,NEISHMX, NSHMX)
      INTEGER N01,M01,L01,NDIST
      INTEGER*4 JUMP, NVAC, IDXV, IDXJUMP, SHIFT,LORDER
      REAL*8 CPV(NVMX)
      REAL*8 CPJ(0:FNEIMX, NVMX)

C     The following variables are kind of self-explanatory.
      REAL*8 CONCB, CONCV, CONCBIN
      REAL*8 TEMP,ENERGY,ENERINIT,ENERIND, ENAV, EN2AV, SIGMA
      REAL*8 SCENERGY, SCENAV, SCEN2AV
      REAL*8 SCENEXCH, TOTPROB, RANPROB, PROB, BASEPROB
      REAL*8 THERMPROB, BALFRE
      REAL*8 BALDIF, BALPRO1, BALPRO2, BALPRO3, BALPRO4, RANDIS
      INTEGER*2 IDTEMP, IDATOMI, IDATOMN, NRNR,ichoice
      REAL*8 NJUMP
      INTEGER*4 BALDIS, NITBAL, BI1, BI2, BI3, EI1, EI2, EI3
      INTEGER*4 NITBAL1, NITBAL2, NITBAL3, NITBAL4
      INTEGER*4 BLAC, ELAC, bran, exran 
      INTEGER*8 INCR,NINI,K1,K2,K3
      REAL*8 R2P

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     TIMIT is the time variable within each iteration.
C     INLTIM - if it is set equal to 0, initialize TIME = 0.0
C        at the beginning of the simulation.
C     TIMVAC : Time elapsed since the last shear
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      REAL*8 TSTEP, TIME, TIMIT, TIMVAC
      INTEGER*4 INLTIM, BVEXCHANGE
      INTEGER II, JJ, KK
      REAL*8 NORMFAC
 
C     Simulation has ITMX iterations containing ISMX steps each.
      INTEGER*4 IPULL, ITMX, ISMX, IT, IS
      INTEGER*4 IRUN, IR, IRRAN(10)
      INTEGER*4 NEIIA, NEIIB, DIFF
      INTEGER*4 NEINA, NEINB
      INTEGER*4 I1, I2, I3, N1, N2, N3, M1, M2, M3
      INTEGER*4 NSH, NEI
      INTEGER*4 ISTOT,IPREP
      INTEGER*4 REACH2, TILT3
C     ALECOM is the name of the random number generating function.
      REAL*8 RTEST,ZERO,RRAN, ALECOM, tiny
      INTEGER*4 LUIN, LUOUT, LUCONIN, LUCON, LUENG
 
      CHARACTER*20 FNAMIN, FNAMOUT, FCONIN, FCON,FENG
 
      DATA NM/12,6,24,12/
      DATA DELTA1N/1,0,0,0/
      DATA ZERO/0.0/
      DATA tiny /1.d-13/
      DATA LUIN,LUOUT,LUCONIN,LUCON,LUENG /10,11,12,13,14/
      DATA FNAMIN,FENG /'tiphirr7in','KMC.ENG'/
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Input section
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      OPEN (LUENG,FILE=FENG,STATUS='NEW')
       
      OPEN(LUIN,FILE=FNAMIN,STATUS='OLD')
      READ (LUIN,'(A)') FNAMOUT
      READ (LUIN,'(A)') FCONIN
      READ (LUIN,'(A)') FCON
 
      write (*,*) fnamout, ' ', fconin, ' ', fcon
      OPEN (UNIT=LUOUT, FILE=FNAMOUT, STATUS='NEW')
      READ (LUIN,*) NBTOTIN, SHIFT, LORDER
      READ (LUIN,*) NVTOT
      IF (NVTOT.GT.1) 
     /   WRITE (*,*) 'WARNING: MORE THAN ONE VACANCY'
      READ (LUIN,*) TEMP
      WRITE (LUOUT, *) 'kT = ', TEMP
 
      READ (LUIN,*) ISHM
      WRITE (LUOUT, *) 'SHELLS FOR A-B INTERACTIONS ', ISHM
      IF (ISHM.GT.2) 
     /   WRITE (*,*) 'WARNING : MORE THAN 2 NEIGHBOR SHELLS'

      DO 50 NSH=1,NSHMX
          EAA(NSH) = 0.0
          EBB(NSH) = 0.0
          EAB(NSH) = 0.0
          EAV(NSH) = 0.0
          EBV(NSH) = 0.0
          WAB(NSH) = 0.0
          WBV(NSH) = 0.0
          WAV(NSH) = 0.0
 50   CONTINUE

      DO 70 NSH=1,ISHM
         READ (LUIN,*) WAB(NSH)
         WRITE(LUOUT,*) 'WAB  OF SHELL ',NSH,' : ',WAB(NSH)
 70   CONTINUE

      READ (LUIN,*) ENCOHNA, ENCOHNB
      READ (LUIN,*) ENVFORA, ENVFORB
      read (luin,*) NU, ENMV
      READ (LUIN,*) ITMX
      READ (LUIN,*) ISMX
      WRITE (LUOUT, *) ITMX, ' ITERATIONS OF ', ISMX, ' STEPS EACH.'
      READ (LUIN,*) baldif
      READ (LUIN,*) NJUMP
      READ (LUIN,*) ICHOICE 

      IF (baldif.lt.0.) baldif = 0.0
      balfre = baldif
      WRITE (LUOUT, *) 'Ballistic diffusion', baldif
      WRITE (LUOUT, *) 'Ballistic jumps', NJUMP,ICHOICE 
      WRITE (*, *) 'Ballistic diffusion', baldif
      WRITE (*, *) 'Ballistic jumps', NJUMP 
      READ (LUIN,*) INLTIM
          

      READ(LUIN,*) IRUN
      do 72 IR = 1, IRUN
         read(luin,*) IRRAN(IR)
 72   continue
      CLOSE (LUIN)
 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C    Initializations
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      CALL NVFCCRH1 (NSHMX, NEISHMX, NVEC, NM)
      DO 200 I1 = LOMIN1, HIMAX1
         MODL1(I1) = MOD(I1, NL1)
         IF (MODL1(I1).LE.0) MODL1(I1) = MODL1(I1) + NL1
 200  CONTINUE
      DO 202 I1 = LOMIN2, HIMAX2
         MODL2(I1) = MOD(I1, NL2)
         IF (MODL2(I1).LE.0) MODL2(I1) = MODL2(I1) + NL2
 202  CONTINUE
      WRITE (*,*) 'NVEC and MODL1, MODL2  done.'

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Determine vacancy-related parameters: WAV, WBV
C     EAA, EBB, EAV, EBV.
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      EAA(1) = 2 * ENCOHNA / FNEIMX
      EBB(1) = 2 * ENCOHNB / FNEIMX
      WAV(1) = 2 * ENVFORA / FNEIMX
      WBV(1) = 2 * ENVFORB / FNEIMX
      EAV(1) = (ENVFORA + ENCOHNA) / FNEIMX
      EBV(1) = (ENVFORB + ENCOHNB) / FNEIMX
c     the saddle point energy
      ENSAD = ENMV + 11*(EAA(1)+EBB(1))/2 + 6*(EAV(1)+EBV(1))
      WRITE (*, '(E14.7,A)') EAA(1), ' EAA - 1'
      WRITE (*, '(E14.7,A)') EBB(1), ' EBB - 1'
      WRITE (*, '(E14.7,A)') EAV(1), ' EAV - 1'
      WRITE (*, '(E14.7,A)') EBV(1), ' EBV - 1'

      DO 210 NSH = 1, NSHMX
          EAB(NSH) = (WAB(NSH) + EAA(NSH) + EBB(NSH)) / 2.0
210   CONTINUE

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Start the loop on the different runs
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      DO 1999 IR = 1, IRUN
C     Initialize the random generator and perform some pulls
      CALL INI 
      DO 240 IPREP= 1, IRRAN(IR)
         RTEST=ALECOM()
 240  CONTINUE
      IPULL=IRRAN(IR)

      NBTOT = NBTOTIN
      OPEN (UNIT=LUCON, FILE=FCON, STATUS='NEW', FORM='UNFORMATTED')
      WRITE (LUOUT, *) 
      WRITE (LUOUT, *) '*********************************'
      WRITE (LUOUT,*) 'RUN NUMBER = ', IR
      WRITE (LUOUT,*) 'NBR OF PULLS BEF. BUILD. CONFIG. =',IPULL
      IF (NBTOT.LT.1) THEN
         WRITE(LUOUT,*) 'READ. CONF. FROM ',FCONIN
      END IF
      CALL VCNFIN5Q(ID1D,NLCB,NBTOT,NVTOT,LUCONIN,FCONIN,SHIFT,
     / LORDER, IPULL, AID, BID, VID, NL1, NL2, TIME, TSTEP, LOCV, NVMX)
      WRITE (LUOUT,*) 'NBR OF PULLS AFT. BUILD. CONFIG. =',IPULL
      IF (INLTIM.EQ.0) TIME = 0.0

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     Construct the neighbor vectors and the MODL array.
C     Initialize the arrays BNEI 
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 
      NATOT = 0
      NBTOT = 0
      NVTOT = 0
      DO 400 I3 = 1, NL2
      DO 400 I2 = 1, NL1
      DO 400 I1 = 1, NL1
         IDATOMI = ID(I1, I2, I3)
         IF (IDATOMI.EQ.AID) NATOT = NATOT + 1
         IF (IDATOMI.EQ.BID) NBTOT = NBTOT + 1
         IF (IDATOMI.EQ.VID) NVTOT = NVTOT + 1
         IF (IDATOMI.EQ.VID) WRITE(*,*) 'VACANCY POSITION:',I1,I2,I3

         DO 350 NSH = 1, NSHMX
            NEIIB = 0
            NEIIA = 0
            DO 330 NEI = 1, NM(NSH)
               N1 = MODL1(I1 + NVEC(1, NEI, NSH))
               REACH2 = I2 + NVEC(2, NEI, NSH)
               N2 = MODL1(REACH2)
               TILT3 = (N2 - REACH2) / NL1
               N3 = MODL2(I3 + TILT3*SHIFT+ NVEC(3, NEI, NSH))
               IF (ID(N1,N2,N3).EQ.BID) NEIIB = NEIIB + 1
 330        CONTINUE
            BNEI(NSH, I1, I2, I3) = NEIIB
 350     CONTINUE
 400  CONTINUE
 
      CONCB = REAL(NBTOT) / REAL(NLCB)
      CONCV = REAL(NVTOT) / REAL(NLCB)
      WRITE (LUOUT, *) 'NUMBER OF B ATOMS IN = ', NBTOTIN
      WRITE (LUOUT, *) 'NUMBER OF B ATOMS   = ', NBTOT
      WRITE (LUOUT, *) 'NUMBER OF VACANCIES = ', NVTOT
      WRITE (*,*) 'Concentration of B atoms   ', CONCB
      WRITE (*,*) 'Concentration of Vacancies ', CONCV
      WRITE (LUOUT,*) 'Concentration of B atoms   ', CONCB
      WRITE (LUOUT,*) 'Concentration of Vacancies ', CONCV
 
C     Initial Energy
      ENERGY = 0.0
      DO 410 I3 = 1, NL2
      DO 410 I2 = 1, NL1
      DO 410 I1 = 1, NL1
         IDATOMI = ID(I1, I2, I3)
         DO 405 NSH = 1, ISHM
            NEIIB = BNEI(NSH, I1, I2, I3)
            NEIIA = NM(NSH) - NEIIB
            IF (IDATOMI.EQ.AID)
     /          ENERGY = ENERGY + NEIIA * EAA(NSH)
     /                         + NEIIB * EAB(NSH) 
            IF (IDATOMI.EQ.BID)
     /          ENERGY = ENERGY + NEIIA * EAB(NSH) 
     /                         + NEIIB * EBB(NSH) 
            IF (IDATOMI.EQ.VID)
     /          ENERGY = ENERGY 
     /                  + NEIIA * (2.0 * EAV(NSH) - EAA(NSH))
     /                  + NEIIB * (2.0 * EBV(NSH) - EAB(NSH))
 405     CONTINUE
 410  CONTINUE
      ENERGY = ENERGY / 2.0 /REAL(NLCB)
      ENERINIT = ENERGY
      SCENERGY = ENERGY / TEMP
 
      WRITE (*,*) 'Initial Energy = ', ENERGY
      WRITE (LUOUT,*) 'Initial Energy = ', ENERGY
      WRITE (LUENG,*) 'Initial Energy = ', ENERGY
 
C     Initialize the vacancy-related arrays.
      DO 420 NVAC = 1, NVMX
         CPV(NVAC) = 0.0
         DO 415 JUMP = 0, FNEIMX
            CPJ(JUMP, NVAC) = 0.0
 415     CONTINUE
 420  CONTINUE
 
      WRITE (LUOUT, *)
      WRITE (LUOUT, '(2(A, E16.9))') ' ', TIME, ' ', ENERGY
      WRITE (*, *)
      WRITE (*, '(2(A, E16.9))') ' ', TIME, ' ', ENERGY

      CALL VCNFOUT5 (ID1D, NLCB, BID, LOCV, NVMX, NVTOT, IPULL,
     /                LUCON, TIME, TSTEP)

C     CALCULATE STRUCTURE FACTOR OF THIS CONFIGRATION
      IT=0
      CALL DIFSUPS(IT,NL1,ID,CONCB,BID,TIME)
      write(*,*) 'Initial Configuration drawn'


CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C     All the initializations are complete now.
C     MONTE CARLO STEPS BEGIN HERE
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      TIMVAC = 0.0
 
      DO 2000 IT = 1, ITMX
         write(*,*)
         write(*,*) '*** Start KMC Step ***', IT

         BVEXCHANGE=0
         NINI=0
         RANDIS=0.0
         ISTOT = 0
         TIMIT = 0.0
         SCENAV = 0.0
         SCEN2AV = 0.0
         nitbal = 0
         nitbal1 = 0
         nitbal2 = 0
         nitbal3 = 0
         nitbal4 = 0
       
         DO 1000 IS = 1, ISMX
            balpro4=0.
            I1 = LOCV(1, 1)
            I2 = LOCV(2, 1)
            I3 = LOCV(3, 1)
C    BASEPROB is the prob. contribution that depends only on the
C    local surrounding of the vacancy under consideration.
            NEIIB = BNEI(1, I1, I2, I3)
            NEIIA = FNEIMX - NEIIB 
            ENBASE = NEIIA*EAV(1) + NEIIB*EBV(1)
 
            DO 475 NEI = 1, FNEIMX
               N1 = MODL1(I1 + NVEC(1, NEI, 1))
               REACH2 = I2 + NVEC(2, NEI, 1)
               N2 = MODL1(REACH2)
               TILT3 = (N2 - REACH2) / NL1
               N3 = MODL2(I3 + TILT3*SHIFT+ NVEC(3, NEI, 1))
               IDATOMN = ID(N1, N2, N3)
 
C     Now, for each neighbor atom, first calculate the part that
C     depends on its vacancy neighbors (only in the first shell).
C     Then calculate the part that depends, again for the first
C     shell, on its atomic neighbors (the A-B contributions).
               ENN1 = 0.0
               IF (IDATOMN.EQ.AID) THEN
                  DO NSH=1, NSHMX	      	 
		            NEINB = BNEI(NSH, N1, N2, N3)
                     NEINA = NM(NSH) - NEINB
                     ENN1 = NEINA*EAA(NSH) + NEINB*EAB(NSH) + ENN1
                  END DO
                  ETOT = ENSAD - ENN1 -(ENBASE-EAA(1))
               ELSE
                  DO NSH=1, NSHMX	      	 
		            NEINB = BNEI(NSH, N1, N2, N3)
                     NEINA = NM(NSH) - NEINB
                     ENN1 = NEINA*EAB(NSH) + NEINB*EBB(NSH) + ENN1
                  END DO
                  ETOT = ENSAD - ENN1 -(ENBASE-EAB(1))
               END IF
C    The probability contribution from A-B bonds is calculated now.
               PROB = NU*DEXP(-ETOT/TEMP) 

C     Cumulative probabilities for the jumps of the vacancy NVAC.
               CPJ(NEI, 1) = CPJ(NEI-1, 1) + PROB
 475        CONTINUE
 
C     Cumulative probability for each vacancy.
            CPV(1) = CPJ(FNEIMX, 1)
 
C     From the total probability, calculate the length of time-step.
            THERMPROB = CPV(1)
            TOTPROB = THERMPROB + BALFRE * NLCB
            TSTEP = 1.0 / TOTPROB
C     Get a random number between 0 and TOTPROB
            RANPROB = TOTPROB * ALECOM()
C    Find whether it's a thermal or ballistic event
            if (ranprob.gt.thermprob) goto 8888
 
            IDXV = 1
            IDXJUMP = 0
            DO 670 JUMP = 1, FNEIMX
               IF (CPJ(JUMP, IDXV).GE.RANPROB) THEN
                  IDXJUMP = JUMP
                  GO TO 671
               END IF
 670        CONTINUE
 671        CONTINUE

C     IDXV and IDXJUMP now represent the particular exchange.
C     Now, identify their locations.
            I1 = LOCV(1, IDXV)
            I2 = LOCV(2, IDXV)
            I3 = LOCV(3, IDXV)

            N1 = MODL1(I1 + NVEC(1, IDXJUMP, 1))
            REACH2 = I2 + NVEC(2, IDXJUMP, 1)
            N2 = MODL1(REACH2)
            TILT3 = (N2 - REACH2) / NL1
            N3 = MODL2(I3 + TILT3*SHIFT+ NVEC(3, IDXJUMP, 1))
            IDATOMN = ID(N1, N2, N3)
            IF (IDATOMN.EQ.VID) WRITE (*,*) 'VACANCIES EXCHANGE?'
                   
C     Perform the exchange.
            ID(I1, I2, I3) = IDATOMN
            ID(N1, N2, N3) = VID
            LOCV(1, IDXV) = N1
            LOCV(2, IDXV) = N2
            LOCV(3, IDXV) = N3

C     Update BNEI only if the exchange is between the vacancy
C     and a ** B ** atom.
            IF (IDATOMN.EQ.BID) THEN
               BVEXCHANGE=BVEXCHANGE+1
               DO 710 NSH = 1, ISHM
               DO 710 NEI = 1, NM(NSH)

                  M1 = MODL1(I1 + NVEC(1, NEI, NSH))
                  REACH2 = I2 + NVEC(2, NEI, NSH)
                  M2 = MODL1(REACH2)
                  TILT3 = (M2 - REACH2) / NL1
                  M3 = MODL2(I3 + TILT3*SHIFT+ NVEC(3, NEI, NSH))
                  BNEI(NSH, M1, M2, M3) = BNEI(NSH, M1, M2, M3) + 1

                  M1 = MODL1(N1 + NVEC(1, NEI, NSH))
                  REACH2 = N2 + NVEC(2, NEI, NSH)
                  M2 = MODL1(REACH2)
                  TILT3 = (M2 - REACH2) / NL1
                  M3 = MODL2(N3 + TILT3*SHIFT+ NVEC(3, NEI, NSH))
                  BNEI(NSH, M1, M2, M3) = BNEI(NSH, M1, M2, M3) - 1
 710           CONTINUE
            END IF
            goto 900 

c   A ballistic event has been selected
 8888       nitbal = nitbal + 1
            blac = 0
            elac = 0
            bran = 1 + int (ALECOM() * DFLOAT(NLCB) * (1.d0-tiny) ) 
            BI3 = (bran - 1) / (NL1 * NL1)
            BI2 = ((bran - BI3 * NL1 * NL1) - 1) / NL1 
            BI1 = bran - BI3 * NL1 * NL1 - BI2 * NL1
            BI3 = BI3 + 1
            BI2 = BI2 + 1
            if (id(bi1,bi2,bi3).eq.vid) then
               do 800  nvac = 1, nvtot
                  if (bi1.eq.locv(1,nvac).and.bi2.eq.locv(2,nvac).
     /               and.bi3.eq.locv(3,nvac)) blac = nvac
 800           continue
            endif

            if(ichoice.eq.1)then
C     RANDOM WALK
               baldis = 1
               ei1=bi1
               ei2=bi2
               ei3=bi3
               do NRNR=1,NJUMP 
                  exran = 1 + int (ALECOM() * DFLOAT(NM(baldis))
     /                 * (1.d0-tiny) )
                  EI1 = MODL1(ei1 + NVEC(1, exran, baldis))
                  REACH2 = ei2 + NVEC(2, exran, baldis)
                  EI2 = MODL1(REACH2)
                  TILT3 = (EI2 - REACH2) / NL1
                  EI3 = MODL2(ei3 + TILT3*SHIFT+ NVEC(3, exran, baldis))
               end do
            else if(ichoice.eq.0)then
c     EXP DISTRIBUTION
               call get_jump(n01,m01,l01,ndist,NJUMP)
               if(ndist.eq.0.or.ndist.gt.11100)   goto 900
               EI1=MODL1(BI1+n01)
               EI2=MODL1(BI2+m01)
               EI3=MODL2(BI3+l01)

            else
               exran = 1 + int (ALECOM() * DFLOAT(NLCB) * (1.d0-tiny))
               EI3 = (exran - 1) / (NL1 * NL1)
               EI2 = ((exran - EI3 * NL1 * NL1) - 1) / NL1
               EI1 = exran - EI3 * NL1 * NL1 - EI2 * NL1
               EI3 = EI3 + 1
               EI2 = EI2 + 1
            end if
                 
            IF (ID(BI1,BI2,BI3).EQ.ID(EI1,EI2,EI3)) goto 900 
            if (id(ei1,ei2,ei3).eq.vid) then
               do 810  nvac = 1, nvtot
                  if (ei1.eq.locv(1,nvac).and.ei2.eq.locv(2,nvac).
     /               and.ei3.eq.locv(3,nvac)) elac = nvac
 810           continue
            endif

cccccccccccccccccccccccccccccccccccccccccccc
    
            NINI=NINI+1
C    Perform the exchange
            IDTEMP = ID(bi1,bi2,bi3)
            ID(bi1,bi2,bi3) = ID(ei1,ei2,ei3)
            ID(ei1,ei2,ei3) = IDTEMP
            If (elac.ne.0) then
               locv(1,elac) = bi1
               locv(2,elac) = bi2
               locv(3,elac) = bi3
            end if
            if (blac.ne.0) then
               locv(1,blac) = ei1
               locv(2,blac) = ei2
               locv(3,blac) = ei3
            end if

C     Update BNEI after a ballistic exchange
            IF (ID(BI1,BI2,BI3).EQ.BID) THEN
               incr = 1
            ELSE IF (ID(EI1,EI2,EI3).EQ.BID) THEN
               incr = -1
            ELSE
               incr = 0
            END IF

            DO 910 NSH = 1, ISHM
            DO 910 NEI = 1, NM(NSH)
               M1 = MODL1(BI1 + NVEC(1, NEI, NSH))
               REACH2 = BI2 + NVEC(2, NEI, NSH)
               M2 = MODL1(REACH2)
               TILT3 = (M2 - REACH2) / NL1
               M3 = MODL2(BI3 + TILT3*SHIFT+ NVEC(3, NEI, NSH))
               BNEI(NSH, M1, M2, M3) = BNEI(NSH, M1, M2, M3) + incr

               M1 = MODL1(EI1 + NVEC(1, NEI, NSH))
               REACH2 = EI2 + NVEC(2, NEI, NSH)
               M2 = MODL1(REACH2)
               TILT3 = (M2 - REACH2) / NL1
               M3 = MODL2(EI3 + TILT3*SHIFT+ NVEC(3, NEI, NSH))
               BNEI(NSH, M1, M2, M3) = BNEI(NSH, M1, M2, M3) - incr
 910        CONTINUE

C     Update the energy, time, etc.
 900        SCENEXCH=0
            SCENERGY = SCENERGY + SCENEXCH
            SCENAV = SCENAV + SCENERGY * TSTEP
            SCEN2AV = SCEN2AV + SCENERGY * SCENERGY * TSTEP
            if(baldif.le.0.) nitbal=1 
            TIME = TIME + TSTEP
            TIMIT = TIMIT + TSTEP
            ISTOT = ISTOT + 1
 1000    CONTINUE

         NORMFAC = 1.0 / TIMIT
         ENERGY = TEMP * SCENERGY
         ENAV = TEMP * SCENAV * NORMFAC
         EN2AV = TEMP * TEMP * SCEN2AV * NORMFAC
         SIGMA = EN2AV - ENAV * ENAV
         WRITE (LUOUT, '(I3,a,I10,2(A, E16.9))') IT, ' times', ISTOT,
     /                ' ', TIME, ' ', TIMIT
         WRITE (LUOUT, '(3(E16.9, A))') ENERGY, ' ', ENAV, ' ', 
     /           TSTEP, ' '
         WRITE (LUOUT, '(2x,A,I9,A,E16.9)') 'Nbrs of ballis :', NITBAL
     /       , '  ballistic frequency :', real(NITBAL)/(TIMIT*NLCB)

         WRITE (*, *)
         WRITE (*, '(I3,a,I10,2(A, E16.9))') IT, ' times', ISTOT,
     /                ' ', TIME, ' ', TIMIT
         WRITE (*, '(3(E16.9, A))') ENERGY, ' ', ENAV, ' ', TSTEP, ' ' 
         WRITE (*, '(2x,A,I9,A,E16.9)') 'Nbrs of ballis :', NITBAL
     /       , '  ballistic frequency :', real(NITBAL)/(TIMIT*NLCB)
         WRITE(*,*) 'NUMBER OF EXCHANGES BETWEEN B AND V:',BVEXCHANGE
         WRITE(LUOUT,*) 'NUMBER OF EXCHANGES BETWEEN B AND V:',
     /        BVEXCHANGE
 
         CALL VCNFOUT5 (ID1D, NLCB, BID, LOCV, NVMX, NVTOT, IPULL,
     /                    LUCON, TIME, TSTEP)
 
C      CALCULATE STRUCTURE FACTOR OF THIS CONFIGRATION
C      ADDED BY JIA YE

         CALL DIFSUPS(IT,NL1,ID,CONCB,BID,TIME)
       
C     CALCULATE THE TOTAL ENERGY
         ENERGY = 0.0
         DO 1200 I3 = 1, NL2
         DO 1200 I2 = 1, NL1
         DO 1200 I1 = 1, NL1
            IDATOMI = ID(I1, I2, I3)
 
            DO 1150 NSH = 1, ISHM
               NEIIB = BNEI(NSH, I1, I2, I3)
               NEIIA = NM(NSH) - NEIIB 
               IF (IDATOMI.EQ.AID)
     /            ENERGY = ENERGY + NEIIA * EAA(NSH)
     /                         + NEIIB * EAB(NSH)
               IF (IDATOMI.EQ.BID)
     /            ENERGY = ENERGY + NEIIA * EAB(NSH) 
     /                         + NEIIB * EBB(NSH) 
               IF (IDATOMI.EQ.VID)
     /            ENERGY = ENERGY 
     /                   + NEIIA * (2.0 * EAV(NSH) - EAA(NSH))
     /                   + NEIIB * (2.0 * EBV(NSH) - EAB(NSH))
 1150       CONTINUE
 1200    CONTINUE

         ENERGY = ENERGY / 2.0 / REAL(NLCB)
         ENERIND = ENERGY - ENERINIT
         ENERINIT = ENERGY
         WRITE (LUENG,*) IT, ENERGY, ENERIND       
 2000 CONTINUE

 1999 CONTINUE 
 
      CLOSE (LUOUT)
      CLOSE (LUCON)
      CLOSE (LUENG)
       
      STOP
      END
 
